@@ -1,4 +1,4 @@
-# Copyright 1997,1998 Spider Boardman.
+# Copyright 1997,2000 Spider Boardman.
 # All rights reserved.
 #
 # Automatic licensing for this software is available.  This software
@@ -11,32 +11,31 @@
 # IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 # WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
+# rcsid: "@(#) $Id: Server.dat,v 1.14 2000/01/19 07:49:40 spider Exp $"
 
 package Net::TCP::Server;
-use 5.004_05;
+use 5.004_04;
 
 use strict;
 #use Carp;
 sub carp { require Carp; goto &Carp::carp; }
 sub croak { require Carp; goto &Carp::croak; }
-use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $AUTOLOAD);
+use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
-my $myclass;
 BEGIN {
-    $myclass = __PACKAGE__;
-    $VERSION = '0.86';
+    $VERSION = '0.93';
+    eval "sub Version () { __PACKAGE__ . ' v$VERSION' }";
 }
-sub Version () { "$myclass v$VERSION" }
 
 #use AutoLoader;
 #use Exporter ();	# we inherit what we need here from Net::Gen
 #use Net::Inet;
 #use Net::Gen;
-use Net::TCP 0.85;
+use Net::TCP 0.93;
 
 
 BEGIN {
-    @ISA = qw(Net::TCP);
+    @ISA = 'Net::TCP';
 
 # Items to export into callers namespace by default
 # (move infrequently used names to @EXPORT_OK below)
@@ -51,6 +50,7 @@ BEGIN {
     %EXPORT_TAGS = (
     ALL		=> [@EXPORT, @EXPORT_OK],
 );
+#    *AUTOLOAD = \$Net::Gen::AUTOLOAD;
 }
 
 # sub AUTOLOAD inherited from Net::Gen (via Net::TCP)
@@ -58,7 +58,7 @@ BEGIN {
 # However, since 5.003_96 will make simple subroutines not inherit AUTOLOAD...
 #sub AUTOLOAD
 #{
-#    $Net::Gen::AUTOLOAD = $AUTOLOAD;
+#    #$Net::Gen::AUTOLOAD = $AUTOLOAD;
 #    goto &Net::Gen::AUTOLOAD;
 #}
 
@@ -69,8 +69,10 @@ BEGIN {
 # Can't autoload routines which we could get without autoloading by
 # inheritance, so new() and init() have to be here.
 
-sub new				# classname, [[hostspec,] service,] [\%params]
+#& new(classname, [[hostspec,] service,] [\%params]) : {$self | undef}
+sub new
 {
+    use attrs qw(locked);
     $_[0]->_trace(\@_,1);
     my ($xclass, @Args) = @_;
     if (@Args == 2 && ref $Args[1] && ref($Args[1]) eq 'HASH' or
@@ -91,21 +93,22 @@ sub new				# classname, [[hostspec,] service,] [\%params]
     $self;
 }
 
-sub init			# $self, [@stuff] ; returns updated $self
+#& init($self, [@stuff]) : {$self | undef}
+sub init
 {
-    use attrs 'locked', 'method';
+    use attrs qw(locked method);
     my ($self, @Args) = @_;
     if (@Args == 2 && ref $Args[1] && ref $Args[1] eq 'HASH' or
 	@Args == 1 and not ref $Args[0]) {
 	unshift(@Args, undef);	# thishost spec
     }
-    return unless $self->_hostport('this',\@Args);
-    return unless $self->SUPER::init;
+    return undef unless $self->_hostport('this',\@Args);
+    return undef unless $self->SUPER::init;
     if ($self->getparam('srcaddrlist') && !$self->isbound) {
-	return unless $self->bind;
+	return undef unless $self->bind;
     }
     if ($self->isbound && !$self->didlisten) {
-	return unless $self->isconnected or $self->listen;
+	return undef unless $self->isconnected or $self->listen;
     }
     $self;
 }
@@ -124,7 +127,6 @@ Net::TCP::Server - TCP sockets interface module for listeners and servers
 
 =head1 SYNOPSIS
 
-    use Socket;			# optional
     use Net::Gen;		# optional
     use Net::Inet;		# optional
     use Net::TCP;		# optional
@@ -144,7 +146,7 @@ itself, rather than just being inherited from C<Net::TCP>,
 C<Net::Inet>, or
 C<Net::Gen>.
 
-=over
+=over 4
 
 =item new
 
@@ -172,7 +174,7 @@ The examples above show the indirect object syntax which many prefer,
 as well as the guaranteed-to-be-safe static method call.  There
 are occasional problems with the indirect object syntax, which
 tend to be rather obscure when encountered.  See
-F<E<lt>URL:http://www.rosat.mpe-garching.mpg.de/mailing-lists/perl-porters/1998-01/msg01674.htmlE<gt>>
+E<lt>URL:http://www.rosat.mpe-garching.mpg.de/mailing-lists/perl-porters/1998-01/msg01674.htmlE<gt>
 for details.
 
 Simple example for server setup:
@@ -213,7 +215,7 @@ module itself.
 
 =head2 Exports
 
-=over
+=over 4
 
 =item default
 
@@ -229,9 +231,19 @@ none
 
 =back
 
+=head1 THREADING STATUS
+
+This module has been tested with threaded perls, and should be as thread-safe
+as perl itself.  (As of 5.005_03 and 5.005_57, that's not all that safe
+just yet.)
+
+=head1 SEE ALSO
+
+Net::TCP(3), Net::Inet(3), Net::Gen(3)
+
 =head1 AUTHOR
 
-Spider Boardman F<E<lt>spider@Orb.Nashua.NH.USE<gt>>
+Spider Boardman E<lt>spider@Orb.Nashua.NH.USE<gt>
 
 =cut
 
