@@ -1,4 +1,4 @@
-# Copyright 1995,1996,1997,1998 Spider Boardman.
+# Copyright 1995,2000 Spider Boardman.
 # All rights reserved.
 #
 # Automatic licensing for this software is available.  This software
@@ -11,37 +11,36 @@
 # IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 # WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
+# rcsid: "@(#) $Id: UDP.dat,v 1.19 2000/01/19 07:49:40 spider Exp $"
 
 package Net::UDP;
-use 5.004_05;			# new minimum Perl version for this package
+use 5.004_04;			# new minimum Perl version for this package
 
 use strict;
 #use Carp;
 sub carp { require Carp; goto &Carp::carp; }
 sub croak { require Carp; goto &Carp::croak; }
-use vars qw($VERSION @ISA $AUTOLOAD);
+use vars qw($VERSION @ISA *AUTOLOAD);
 
-my $myclass;
 BEGIN {
-    $myclass = __PACKAGE__;
-    $VERSION = '0.86';
+    $VERSION = '0.93';
+    eval "sub Version () { __PACKAGE__ . ' v$VERSION' }";
 }
-sub Version () { "$myclass v$VERSION" }
 
 use AutoLoader;
 
-use Net::Inet 0.85;
-use Net::Gen 0.85;
-use Socket qw(!/^[a-z]/ !SOMAXCONN);
+use Net::Inet 0.93;
+use Net::Gen 0.93 ':sockvals', ':families';
 
 BEGIN {
-    @ISA = qw(Net::Inet);
+    @ISA = 'Net::Inet';
+    *AUTOLOAD = \$Net::Gen::AUTOLOAD;
 }
 
 # Cheat on AUTOLOAD inheritance.
 sub AUTOLOAD
 {
-    $Net::Gen::AUTOLOAD = $AUTOLOAD;
+    #$Net::Gen::AUTOLOAD = $AUTOLOAD;
     goto &Net::Gen::AUTOLOAD;
 }
 
@@ -61,26 +60,28 @@ my %Keys;			# for only calling registration routines once
 
 sub new
 {
+    use attrs qw(locked);
     my($class,@args) = @_;
     my $self = $class->SUPER::new(@args);
     $class = ref $class if ref $class;
     if ($self) {
 	if (%Keys) {
-	    ${*$self}{Keys} = { %Keys } ;
+	    $ {*$self}{Keys} = { %Keys } ;
 	}
 	else {
 	    $self->register_param_keys(\@Keys);
 	    $self->register_param_handlers(\%CodeKeys);
-	    %Keys = %{ ${*$self}{Keys} } ;
+	    %Keys = %{ $ {*$self}{Keys} } ;
 	}
 	# no new sockopts for UDP?
 	# set our required parameters
 	$self->setparams({type => SOCK_DGRAM,
 			  proto => IPPROTO_UDP,
 			  IPproto => 'udp',
+			  netgen_fakeconnect => 1,
 			  unbuffered_output => 0,
 			  unbuffered_input => 0}, -1);
-	if ($class eq $myclass) {
+	if ($class eq __PACKAGE__) {
 	    unless ($self->init(@args)) {
 		local $!;	# protect returned errno value
 		undef $self;	# against excess closes in perl core
@@ -91,11 +92,12 @@ sub new
     $self;
 }
 
-sub _addrinfo			# $this, $sockaddr, [numeric_only]
+#& _addrinfo($this, $sockaddr, [numeric_only]) : @list
+sub _addrinfo
 {
     my($this,@args,@r) = @_;
     @r = $this->SUPER::_addrinfo(@args);
-    unless(!@r or ref($this) or $r[2] ne $r[3]) {
+    unless(!@r or $args[1] or ref($this) or $r[2] ne $r[3]) {
 	$this = getservbyport(htons($r[3]), 'udp');
 	$r[2] = $this if defined $this;
     }
@@ -118,7 +120,6 @@ Net::UDP - UDP sockets interface module
 
 =head1 SYNOPSIS
 
-    use Socket;			# optional
     use Net::Gen;		# optional
     use Net::Inet;		# optional
     use Net::UDP;
@@ -135,7 +136,7 @@ The following methods are provided by the C<Net::UDP> module
 itself, rather than just being inherited from C<Net::Inet> or
 C<Net::Gen>.
 
-=over 6
+=over 4
 
 =item new
 
@@ -165,7 +166,7 @@ The examples above show the indirect object syntax which many prefer,
 as well as the guaranteed-to-be-safe static method call.  There
 are occasional problems with the indirect object syntax, which
 tend to be rather obscure when encountered.  See
-F<E<lt>URL:http://www.rosat.mpe-garching.mpg.de/mailing-lists/perl-porters/1998-01/msg01674.htmlE<gt>>
+E<lt>URL:http://www.rosat.mpe-garching.mpg.de/mailing-lists/perl-porters/1998-01/msg01674.htmlE<gt>
 for details.
 
 =item PRINT
@@ -206,7 +207,7 @@ then this method does line-buffering of its input as defined by
 the current setting of the $/ variable.  If the
 <unbuffered_input> parameter is true, then the input records will
 be exact recv() datagrams, disregarding the setting of the $/
-variable.  Note that invoking the C<READLINE> method in array
+variable.  Note that invoking the C<READLINE> method in list
 context is likely to hang, since UDP sockets typically don't
 return EOF.
 
@@ -235,7 +236,7 @@ of the $/ variable.  The default is false, which causes the C<READLINE>
 interface to return lines split at boundaries as appropriate for $/.
 (The C<READLINE> method for tied filehandles is the C<E<lt>FHE<gt>>
 operation.)  Note that calling the C<READLINE> method
-in array context is likely to hang.
+in list context is likely to hang.
 
 =item unbuffered_output
 
@@ -273,21 +274,31 @@ machines.
 
 Each assignment to the tied scalar is really a call to the C<put>
 method (via the C<STORE> method), and each read from the tied
-scalar is really a call to the C<getline> method (via the
+scalar is really a call to the C<READLINE> method (via the
 C<FETCH> method).
 
 =head2 TIEHANDLE support
 
-As inherited from C<Net::Inet> and C<Net::Gen>, with the additions of
+As inherited from C<Net::Inet> and C<Net::Gen>, with the addition of
 unbuffered I/O options for the C<READLINE> and C<PRINT> methods.
 
 =head2 Exports
 
 none
 
+=head1 THREADING STATUS
+
+This module has been tested with threaded perls, and should be as thread-safe
+as perl itself.  (As of 5.005_03 and 5.005_57, that's not all that safe
+just yet.)
+
+=head1 SEE ALSO
+
+Net::Inet(3), Net::Gen(3)
+
 =head1 AUTHOR
 
-Spider Boardman F<E<lt>spider@Orb.Nashua.NH.USE<gt>>
+Spider Boardman E<lt>spider@Orb.Nashua.NH.USE<gt>
 
 =cut
 
@@ -295,17 +306,19 @@ Spider Boardman F<E<lt>spider@Orb.Nashua.NH.USE<gt>>
 
 #any real autoloaded methods go after this line
 
-sub _setbuf_unbuf		# $self, $param, $newvalue;
-{				# returns '' or carp string
+#& _setbuf_unbuf($self, $param, $newvalue) : {'' | "carp string"}
+sub _setbuf_unbuf
+{
     my ($self,$what,$newval) = @_;
     $self->setparams({unbuffered_input => $newval,
 		      unbuffered_output => $newval});
     '';
 }
 
-sub PRINT			# $self, @args; returns boolean OKness
+#& PRINT($self, @args) : OKness
+sub PRINT
 {
-    use attrs 'locked', 'method';
+    use attrs qw(locked method);
     my $self = shift;
     if ($self->getparam('unbuffered_output')) {
 	$self->send(join $, , @_);
@@ -315,9 +328,10 @@ sub PRINT			# $self, @args; returns boolean OKness
     }
 }
 
-sub READLINE		# $self; returns buffer or array of buffers
-{				# barfs if called unbuffered in array context
-    use attrs 'locked', 'method';
+#& READLINE($self) : $line | undef || @lines
+sub READLINE
+{
+    use attrs qw(locked method);
     my $whoami = $_[0]->_trace(\@_,5);
     carp "Excess arguments to ${whoami}, ignored" if @_ > 1;
     my $self = shift;
